//
//  HWShareViewController.m
//  VclustersGemini
//
//  Created by 余衡武 on 2021/11/16.
//  Copyright © 2021 APPLE. All rights reserved.
//

#import "HWShareViewController.h"

//微信分享头文件
#import "WXApi.h"
#import "WXApiObject.h"

//QQ分享头文件
#import <TencentOpenAPI/TencentOAuth.h>
#import <TencentOpenAPI/QQApiInterface.h>
#import <TencentOpenAPI/QQApiInterfaceObject.h>

#import "HWBuryPointModel.h"
#import "BuryPoint.h"
#import <SDWebImage.h>
#import "UseAccountManage.h"

@interface HWShareViewController ()
{
    UIImageView *imgView;
}

@property (weak, nonatomic) IBOutlet UIView *bgView1;
@property (weak, nonatomic) IBOutlet UIView *bgView2;
@property (weak, nonatomic) IBOutlet UIView *bgView3;
@property (weak, nonatomic) IBOutlet UIView *linkView;
@property (weak, nonatomic) IBOutlet UIView *circleView;
@property (weak, nonatomic) IBOutlet UIView *WXView;
@property (weak, nonatomic) IBOutlet UIView *QQView;

@property (weak, nonatomic) IBOutlet UILabel *topLabelLabel;
@property (weak, nonatomic) IBOutlet UIButton *cancelBtn;

@end

@implementation HWShareViewController
@synthesize delegate;
@synthesize isNeedHidenLink;


#pragma mark 生命周期
- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self.toolBar setHidden:YES];
    [self.navigationBar setHidden:YES];
    [self.bgView1 setBackgroundColor:[UIColor clearColor]];
    self.navBarBGView.hidden = YES;
    self.bgView3.backgroundColor = HWFFFFFFColor;
    [self.view setBackgroundColor:[UIColor clearColor]];
    self.bgView2.backgroundColor = HWFFFFFFColor;
    self.bgView2.layer.cornerRadius = 8;
    self.bgView2.clipsToBounds = YES;
    
    [self.linkView addGestureRecognizer:[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(linkViewClick)]];
    [self.circleView addGestureRecognizer:[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(circleViewClick)]];
    [self.WXView addGestureRecognizer:[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(WXViewClick)]];
    [self.QQView addGestureRecognizer:[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(QQViewClick)]];
    
    if (isNeedHidenLink){
        self.linkView.hidden = YES;
        
        [self.circleView removeFromSuperview];
        [self.bgView3 addSubview:self.circleView];
        [self.circleView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.centerX.mas_equalTo(self.bgView3.mas_centerX);
            make.width.mas_equalTo(97.f);
            make.bottom.mas_equalTo(0);
            make.height.mas_equalTo(73.f);
        }];
        
        self.QQView.hidden = YES;
//        [self.QQView removeFromSuperview];
//        [self.bgView3 addSubview:self.QQView];
//        [self.QQView mas_remakeConstraints:^(MASConstraintMaker *make) {
//            make.right.mas_equalTo(-20);
//            make.width.mas_equalTo(97.f);
//            make.bottom.mas_equalTo(0);
//            make.height.mas_equalTo(73.f);
//        }];
        
        self.WXView.hidden = YES;
//        [self.WXView removeFromSuperview];
//        [self.bgView3 addSubview:self.WXView];
//        [self.WXView mas_remakeConstraints:^(MASConstraintMaker *make) {
//            make.centerX.mas_equalTo(self.bgView3.mas_centerX);
//            make.width.mas_equalTo(97.f);
//            make.bottom.mas_equalTo(0);
//            make.height.mas_equalTo(73.f);
//        }];
    }
    
    [self.bgView1 addGestureRecognizer:[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(cancel)]];
    
//    self.model.shareUrl = @"http://test.androidscloud.com/h5/microserviceUserH5/vcloud/share.html?id=323";
    if (self.model.ID != 0) {
        self.model.shareUrl = [NSString stringWithFormat:@"%@/h5/microserviceUserH5/vcloud/share.html?id=%ld", CloudService,self.model.ID];
        self.model.topImage = [NSString stringWithFormat:@"%@/document/newFile/download/0/%@?fileKey=%@", FileService, Const_File_Access_Key,self.model.topImage];
    }
    imgView = [[UIImageView alloc] init];
    
    [_cancelBtn setTitleColor:HW13B2EBColor forState:(UIControlStateNormal)];
    [_topLabelLabel setTextColor:HWCFD1D4Color];
}

#pragma mark 点击事件
- (void)linkViewClick {
//    HLog(@"复制链接");
    [self dismissViewControllerAnimated:YES completion:^{
        
        UIPasteboard *pb = [UIPasteboard generalPasteboard];
        pb.string = self.model.shareUrl;
        [[iToast makeText:NSLocalizedString(@"share_copy_paste_link",nil)] show];
    }];
}

- (void)circleViewClick {
//    HLog(@"朋友圈");
    [self dismissViewControllerAnimated:YES completion:^{
        [self shareWechatMoments:self.model.title content:self.model.introduction imgUrl:self.model.topImage url:self.model.shareUrl];
    }];
}

- (void)WXViewClick {
//    HLog(@"微信好友");
    [self dismissViewControllerAnimated:YES completion:^{
        [self shareWechatFriend:self.model.title content:self.model.introduction imgUrl:self.model.topImage url:self.model.shareUrl];
    }];
}

// topImage = file-center/fileOperate/getImage?id=632901629;

- (void)QQViewClick {
//    HLog(@"QQ");
    [self dismissViewControllerAnimated:YES completion:^{
        [self shareQQWith:self.model.title content:self.model.introduction imgUrl:self.model.topImage
                      url:self.model.shareUrl];
    }];
}

- (IBAction)cancelBtnClick:(UIButton *)sender {
    HLog(@"取消");
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

- (void)cancel {
    HLog(@"取消");
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

#pragma mark 社会化分享
- (void)shareQQWith:(NSString *)title content:(NSString *)content imgUrl:(NSString *)imgUrl url:(NSString *)url
{
    if(![QQApiInterface isSupportShareToQQ])
    {
        [[iToast makeText:NSLocalizedString(@"share_no_qq",nil)] show];
        return;
    }
    
    //哪一个默认的图片 获取图片出错时显示
    __block UIImage *shareImage = [UIImage imageNamed:@"AppIcon"];
        [self showNewIndicatorWithCanBack:YES canTouch:NO];
        [imgView sd_setImageWithURL:[NSURL URLWithString:imgUrl]
                                    placeholderImage:[UIImage imageNamed:@"AppIcon"]
                                            options:SDWebImageAllowInvalidSSLCertificates
                                completed:^(UIImage * _Nullable image, NSError * _Nullable error, SDImageCacheType cacheType, NSURL * _Nullable imageURL) {
            [self removeNewIndicator];
            if(image)
            {
                QQApiNewsObject *newsObj = [QQApiNewsObject
                objectWithURL:[NSURL URLWithString:url]
                title:title
                description:content
                previewImageData:UIImagePNGRepresentation(image)];
                SendMessageToQQReq *req = [SendMessageToQQReq reqWithContent:newsObj];
                //将内容分享到qq
                QQApiSendResultCode sent = [QQApiInterface sendReq:req];
                if (sent != 0) {
                    [[iToast makeText:NSLocalizedString(@"share_share_faile_qq",nil)] show];
                    NSLog(@"QQ好友分享失败");
                }
                else {
                    NSLog(@"QQ好友分享成功");
                    if ([self->delegate respondsToSelector:@selector(haveShowOk)]){
                        [self->delegate haveShowOk];
                    }
                }
            }
            else {
                QQApiNewsObject *newsObj = [QQApiNewsObject
                objectWithURL:[NSURL URLWithString:url]
                title:title
                description:content
                previewImageData:UIImagePNGRepresentation(shareImage)];
                SendMessageToQQReq *req = [SendMessageToQQReq reqWithContent:newsObj];
                //将内容分享到qq
                QQApiSendResultCode sent = [QQApiInterface sendReq:req];
                if (sent != 0) {
                    [[iToast makeText:NSLocalizedString(@"share_share_faile_qq",nil)] show];
                    NSLog(@"QQ好友分享失败");
                }
                else {
                    NSLog(@"QQ好友分享成功");
                    if ([self->delegate respondsToSelector:@selector(haveShowOk)]){
                        [self->delegate haveShowOk];
                    }
                }
            }
        }];
}

- (void)shareWechatFriend:(NSString *)title content:(NSString *)content imgUrl:(NSString *)imgUrl url:(NSString *)url
{
    if(![WXApi isWXAppInstalled])
    {
        [[iToast makeText:NSLocalizedString(@"share_no_wx",nil)] show];
        return;
    }
    
    //哪一个默认的图片 获取图片出错时显示
    __block UIImage *shareImage = [UIImage imageNamed:@"AppIcon"];
    [self showNewIndicatorWithCanBack:YES canTouch:NO];
        [imgView sd_setImageWithURL:[NSURL URLWithString:imgUrl]
                                  placeholderImage:[UIImage imageNamed:@"AppIcon"]
                                            options:SDWebImageAllowInvalidSSLCertificates
                                completed:^(UIImage * _Nullable image, NSError * _Nullable error, SDImageCacheType cacheType, NSURL * _Nullable imageURL) {
            [self removeNewIndicator];
            if(image)
            {
                shareImage = image;
                WXWebpageObject *webpageObject = [WXWebpageObject object];
                webpageObject.webpageUrl = url;
                WXMediaMessage *message = [WXMediaMessage message];
                message.title = title;
                message.description =  content;
                [message setThumbImage:shareImage];
                message.mediaObject = webpageObject;
                SendMessageToWXReq *req = [[SendMessageToWXReq alloc] init];
                req.bText = NO;
                req.message = message;
                req.scene = WXSceneSession;
                [WXApi sendReq:req completion:^(BOOL success) {
                    if (success == NO) {
                        NSLog(@"微信好友分享失败");
                        [[iToast makeText:NSLocalizedString(@"share_share_faile_wx",nil)] show];
                    }else {
                        NSLog(@"微信好友分享成功");
                        if ([self->delegate respondsToSelector:@selector(haveShowOk)]){
                            [self->delegate haveShowOk];
                        }
                    }
                }];
            }
            else {
                WXWebpageObject *webpageObject = [WXWebpageObject object];
                webpageObject.webpageUrl = url;
                WXMediaMessage *message = [WXMediaMessage message];
                message.title = title;
                message.description =  content;
                [message setThumbImage:shareImage];
                message.mediaObject = webpageObject;
                SendMessageToWXReq *req = [[SendMessageToWXReq alloc] init];
                req.bText = NO;
                req.message = message;
                req.scene = WXSceneSession;
                [WXApi sendReq:req completion:^(BOOL success) {
                    if (success == NO) {
                        NSLog(@"微信好友分享失败");
                        [[iToast makeText:NSLocalizedString(@"share_share_faile_wx",nil)] show];
                    }else {
                        NSLog(@"微信好友分享成功");
                        if ([self->delegate respondsToSelector:@selector(haveShowOk)]){
                            [self->delegate haveShowOk];
                        }
                    }
                }];
            }
        }];
}

- (void)shareWechatMoments:(NSString *)title content:(NSString *)content imgUrl:(NSString *)imgUrl url:(NSString *)url
{
    if(![WXApi isWXAppInstalled])
    {
        [[iToast makeText:NSLocalizedString(@"share_no_wx",nil)] show];
        return;
    }
    
    //哪一个默认的图片 获取图片出错时显示
    __block UIImage *shareImage = [UIImage imageNamed:@"AppIcon"];
    [self showNewIndicatorWithCanBack:YES canTouch:NO];
        [imgView sd_setImageWithURL:[NSURL URLWithString:imgUrl]
                                  placeholderImage:[UIImage imageNamed:@"AppIcon"]
                                            options:SDWebImageAllowInvalidSSLCertificates
                                completed:^(UIImage * _Nullable image, NSError * _Nullable error, SDImageCacheType cacheType, NSURL * _Nullable imageURL) {
            [self removeNewIndicator];
            if(image)
            {
                shareImage = image;
                WXWebpageObject *webpageObject = [WXWebpageObject object];
                webpageObject.webpageUrl = url;
                WXMediaMessage *message = [WXMediaMessage message];
                message.title =  title;
                message.description =  content;
                [message setThumbImage:shareImage];
                message.mediaObject = webpageObject;
                SendMessageToWXReq *req = [[SendMessageToWXReq alloc] init];
                req.bText = NO;
                req.message = message;
                req.scene = WXSceneTimeline;
                [WXApi sendReq:req completion:^(BOOL success) {
                    NSLog(@"微信朋友圈分享");
                    if (success == NO) {
                        [[iToast makeText:NSLocalizedString(@"share_share_faile_wx_moment",nil)] show];
                        NSLog(@"微信朋友圈分享失败");
                    }
                    else{
                        NSLog(@"微信朋友圈分享成功");
                        if ([self->delegate respondsToSelector:@selector(haveShowOk)]){
                            [self->delegate haveShowOk];
                        }
                    }
                }];
            }else {
                WXWebpageObject *webpageObject = [WXWebpageObject object];
                webpageObject.webpageUrl = url;
                WXMediaMessage *message = [WXMediaMessage message];
                message.title =  title;
                message.description =  content;
                [message setThumbImage:shareImage];
                message.mediaObject = webpageObject;
                SendMessageToWXReq *req = [[SendMessageToWXReq alloc] init];
                req.bText = NO;
                req.message = message;
                req.scene = WXSceneTimeline;
                [WXApi sendReq:req completion:^(BOOL success) {
                    NSLog(@"微信朋友圈分享");
                    if (success == NO) {
                        [[iToast makeText:NSLocalizedString(@"share_share_faile_wx_moment",nil)] show];
                        NSLog(@"微信朋友圈分享失败");
                    }
                    else{
                        NSLog(@"微信朋友圈分享成功");
                        if ([self->delegate respondsToSelector:@selector(haveShowOk)]){
                            [self->delegate haveShowOk];
                        }
                    }
                }];
            }
        }];
}

@end
