//
//  QQLoginManager.m
//  VclustersGemini
//
//  Created by 余衡武 on 2020/5/13.
//  Copyright © 2020 APPLE. All rights reserved.
//

#import "QQLoginManager.h"
#import "iPhone.h"

static QQLoginManager *shareInstance = nil;

@implementation QQLoginManager

+ (instancetype)shareInstance {
    
    static dispatch_once_t onceToken;
    
    dispatch_once(&onceToken, ^{
        shareInstance = [[QQLoginManager alloc] init];
        
    });
    
    return shareInstance;
}

- (void)authorizeQQLogin {
    //发起授权登录请求
    NSArray *permissions = [NSArray arrayWithObjects:@"get_user_info", @"add_share",@"get_simple_userinfo",@"add_t", nil];
    [[QQLoginManager shareInstance].tencentOAuth authorize:permissions inSafari:NO];
}

#pragma mark -- TencentSessionDelegate
//登录完成调用
- (void)tencentDidLogin
{
    if (self.tencentOAuth.accessToken && 0 != [self.tencentOAuth.accessToken length]) {
        //这里可获取accessToken,Access Token凭证，用于后续访问各开放接口
        HLog(@"accessToken %@",self.tencentOAuth.accessToken);
        //这里可获取openID,openId是用户授权登录后对该用户的唯一标识
        HLog(@"openId %@",self.tencentOAuth.openId);
    }else {
        HLog(@"登录不成功没有获取accesstoken");
        return;
    }
      
    [self.loginParams removeAllObjects];
    
    //这里可获取accessToken,Access Token凭证，用于后续访问各开放接口
    [self.loginParams setObject:[QQLoginManager shareInstance].tencentOAuth.accessToken forKey:@"accessToken"];
    //这里可获取openID,openId是用户授权登录后对该用户的唯一标识
    [self.loginParams setObject:[QQLoginManager shareInstance].tencentOAuth.openId forKey:@"openId"];
    [self.loginParams setObject:[QQLoginManager shareInstance].tencentOAuth.openId forKey:@"unionid"];
    [self.loginParams setObject:[QQLoginManager shareInstance].tencentOAuth.openId forKey:@"uid"];
    NSDate *expiresDate = [QQLoginManager shareInstance].tencentOAuth.expirationDate;
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"ss"];//设置需要转换的格式
    NSString *expiresTime = [dateFormatter stringFromDate:expiresDate];
    [self.loginParams setObject:expiresTime forKey:@"expiresIn"];
    [self.loginParams setObject:@"refresh_token" forKey:@"refreshToken"];
    [self.loginParams setObject:[iPhone phoneDeviceUuid] forKey:@"uuid"];
    [self.loginParams setObject:[iPhone phoneDeviceType] forKey:@"diskName"];
    [self.loginParams setObject:[iPhone phoneDeviceIPAddress] forKey:@"ipAddr"];
    [self.loginParams setObject:@"2" forKey:@"authorizationType"];
    [self.loginParams setObject:@"2" forKey:@"client"];
    [self.loginParams setObject:QQAPPid forKey:@"appId"];
    
    //获取用户信息
    int result1 = [self.tencentOAuth getUserInfo];
    if (result1) {
        HLog(@"QQ登录 getUserInfo 成功");
    }else {
        HLog(@"QQ登录 getUserInfo 失败");
    }
    //获取UnindID
    int result2 = [self.tencentOAuth RequestUnionId];
    if (result2) {
        HLog(@"QQ登录 RequestUnionId 成功");
    }else {
        HLog(@"QQ登录 RequestUnionId 失败");
    }
    
}
  
  
//非网络错误导致登录失败：
-(void)tencentDidNotLogin:(BOOL)cancelled {
    if (cancelled) {
        HLog(@"用户取消登录");
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [[iToast makeText:@"用户取消了QQ登录"] show];
        });
    }else {
        HLog(@"登录失败");
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [[iToast makeText:@"QQ登录失败 请稍后再试"] show];
        });
    }
}
  
// 网络错误导致登录失败：
- (void)tencentDidNotNetWork
{
    HLog(@"tencentDidNotNetWork");
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [[iToast makeText:@"网络异常 请稍后再试"] show];
    });
}
  
- (void)getUserInfoResponse:(APIResponse *)response
{
    HLog(@"respons:%@",response.jsonResponse);
    
    [self.loginParams setObject:response.jsonResponse[@"figureurl_2"] forKey:@"iconurl"];
    [self.loginParams setObject:response.jsonResponse[@"nickname"] forKey:@"name"];
    NSString *sex = [NSString stringWithFormat:@"%@", response.jsonResponse[@"gender"]];
    [self.loginParams setObject:sex forKey:@"gender"];
    
    if (self.delegate && [self.delegate respondsToSelector:@selector(getQQLoginParamsFromManager)]) {
        [self.delegate getQQLoginParamsFromManager];
    }
}

//#pragma mark <TencentSessionDelegate>
///**
// * 退出登录的回调
// */
//- (void)tencentDidLogout {
//    HLog(@"退出QQ登录")
//}
//
///**
// * 因用户未授予相应权限而需要执行增量授权。
// */
//- (BOOL)tencentNeedPerformIncrAuth:(TencentOAuth *)tencentOAuth withPermissions:(NSArray *)permissions {
//    return YES;
//}
//
///**
// * [该逻辑未实现]因token失效而需要执行重新登录授权。
// */
//- (BOOL)tencentNeedPerformReAuth:(TencentOAuth *)tencentOAuth {
//    return YES;
//}
//
///**
// * 用户通过增量授权流程重新授权登录，token及有效期限等信息已被更新。
// */
//- (void)tencentDidUpdate:(TencentOAuth *)tencentOAuth {
//
//}
//
///**
// * 用户增量授权过程中因取消或网络问题导致授权失败
// */
//- (void)tencentFailedUpdate:(UpdateFailType)reason {
//
//}
//
///**
// * 获取用户个人信息回调
// * \param response API返回结果，具体定义参见sdkdef.h文件中\ref APIResponse
// * \remarks 正确返回示例: \snippet example/getUserInfoResponse.exp success
// *          错误返回示例: \snippet example/getUserInfoResponse.exp fail
// */
////- (void)getUserInfoResponse:(APIResponse*) response {
////
////}
//
///**
// * 社交API统一回调接口
// */
//- (void)responseDidReceived:(APIResponse*)response forMessage:(NSString *)message {
//
//}

#pragma mark 懒加载
- (NSMutableDictionary *)loginParams {
    if (!_loginParams) {
        _loginParams = [NSMutableDictionary dictionary];
    }
    return _loginParams;;
}

@end
