//
//  CloudPhoneModel.h
//  VclustersGemini
//
//  Created by APPLE on 2020/7/27.
//  Copyright © 2020 APPLE. All rights reserved.
//

#import "SuperModel.h"
/**在需要存储的model类中,添加BGFMDB.h头文件,本类就具有了BGFMDB存储功能.*/
#import "BGFMDB.h"

NS_ASSUME_NONNULL_BEGIN

/**
`云手机相关
@discussion screenShareStatus     屏幕分享状态 0屏幕分享中  1分享暂停  2未分享  3屏幕授权分享中
@discussion diskName                   云手机名称
@discussion authPhone                 是否为授权云手机 0非授权手机(自己的手机)  1别人授权的手机
@discussion sn                              云手机序列号
@discussion readme                      云手机过期相关提醒
@discussion buyVipType               云手机类型  VIP星动云手机  SVIP星曜云手机 STAR 唔即云手机
@discussion phoneAuthStatus      云手机授权状态 0未授权   1已授权
@discussion phoneId                     云手机Id
@discussion exIp                           云手机ip地址
@discussion exPort                       云手机端口号
@discussion myOrderNum            云手机购买/续费订单号
@discussion validTime                   云手机有效时间(精确到分)
@discussion username                  云手机所属用户名
@discussion areaStatus                 是否正在进行站点迁移
@discussion areaId                        站点ID
@discussion androidVersion         安卓系统版本 7、10
*/
@interface CloudPhoneModel : SuperModel

/**站点迁移状态*/
typedef NS_ENUM(NSInteger, SiteSwitchState) {
    SiteSwitchStateFail     = 0,  // 0、切换失败
    SiteSwitchStateSuccess  = 1,  // 1、切换成功
    SiteSwitchStateMoving   = 2,  // 2、切换中
    SiteSwitchStateBacking  = 3,  // 3、回滚中
    SiteSwitchStateNormal   = 4,  // 4、正常状态
};

/**
* @brief  云游戏下载 云机配置推荐  1.星动 2.星曜 3.唔即 4.唔即pro
*/
@property (nonatomic,assign) int cloudMachine;//

/**
* @brief  云游戏下载 配置推荐isRecommended 0 不推荐 1 推荐
*/
@property (nonatomic,assign) BOOL isRecommended;

/**
* @brief  屏幕分享状态 0屏幕分享中  1分享暂停  2未分享  3屏幕授权分享中
*/
@property (nonatomic,assign) NSInteger screenShareStatus;

/**
* @brief  云手机名称
*/
@property (nonatomic,copy) NSString *diskName;

/**
* @brief  云手机购买时间
*/
@property (nonatomic,copy) NSString *ctime;

/**
* @brief  是否为授权云手机 0非授权手机(自己的手机)  1别人授权的手机
*/
@property (nonatomic,assign) NSInteger authPhone;

/**
* @brief  云手机序列号
*/
@property (nonatomic,copy) NSString *sn;

/**
* @brief  云手机过期相关提醒
*/
@property (nonatomic,copy) NSString *readme;

/**
* @brief  云手机类型  VIP星动云手机  SVIP星曜云手机  STAR 唔即云手机 STARPRO 唔即PRO云手机
*/
@property (nonatomic,copy) NSString *buyVipType;

/**
* @brief  云手机授权状态 0未授权   1已授权
*/
@property (nonatomic,assign) NSInteger phoneAuthStatus;

/**
* @brief  云手机Id
*/
@property (nonatomic,assign) NSInteger phoneId;


/**
* @brief  云手机分配rbd后的 id 用于判断用户是否有查看过云手机 为0就是首次进入
*/
@property (nonatomic,assign) long cardId;

/**
* @brief  云手机ip地址
*/
@property (nonatomic,copy) NSString *internetIp;

/**
* @brief 机房外网ip （ping地址）
*/
@property (nonatomic,copy) NSString *roomInternetIp;


/**
* @brief  音视频外网端口
*/
@property (nonatomic,copy) NSString *internetVideoPort;

/**
* @brief  云手机购买/续费订单号
*/
@property (nonatomic,copy) NSString *myOrderNum;

/**
* @brief  云手机有效时间(精确到分)
*/
@property (nonatomic,assign) NSInteger validTime;

/**
* @brief  站点迁移剩余时间(精确到分)
*/
@property (nonatomic,assign) NSInteger switchTime;

/**
* @brief  云手机所属用户名
*/
@property (nonatomic,copy) NSString *username;

/**
* @brief  是否正在进行站点迁移 0、切换失败 1、切换成功 2、切换中 3、回滚中 4、正常状态
*/
@property (nonatomic,assign) SiteSwitchState areaStatus;

/**
* @brief  站点迁移是否排队  -1 排队中
*/
@property (nonatomic,assign) NSInteger moveStatus;

/**
* @brief  站点ID
*/
@property (nonatomic,assign) NSInteger areaId;

/**
* @brief  站点名称
*/
@property (nonatomic,copy) NSString *areaName;

/**
* @brief  是否具有超分权限 0没有  1有
*/
@property (nonatomic,assign) NSInteger overStatus;

/**
* @brief  设备扩容大小，单位G
*/
@property (nonatomic, assign) NSInteger rbdExtendSize;

/**
* @brief  虚拟按键状态 1：开启 0：关闭
*/
@property(nonatomic, assign)NSInteger virtualKeyStatus;

/**
* @brief  专业模式状态 0：常规 1：专业
*/
@property(nonatomic, assign)NSInteger usageMode;

/**
* @brief  ROOT按键状态 1：开启 0：关闭
*/
@property(nonatomic, assign)NSInteger rootStatus;

/**
* @brief  使用习惯 1：左手 2：右手
*/
@property(nonatomic, assign)NSInteger usageHabit;

/**
*
* @brief  云手机是否被选中-自定义属性
*/
@property (nonatomic, assign) BOOL isSelect;

/**
* @brief vpnStatus 1：开启    0：关闭
*/
@property (nonatomic, assign) NSInteger vpnStatus;

/**
* @brief startGetRBDTime 开启rbd时间
*/
@property (nonatomic, assign) double startGetRBDTime;


@property (nonatomic, assign) NSInteger userCardId;

/**
* @brief chaofenPhoneIndex 开启超分设备下标
*/
@property (nonatomic, assign) NSInteger chaofenPhoneIndex;

/**
* @brief chaofenPhoneIndex 开启超分设备结果状态
*/
@property (nonatomic, assign) BOOL chaofenResult;

/**
* @brief freeTrial 标记是否为免费试用云手机
*/
@property (nonatomic, copy) NSString   *freeTrial;//免费试用 : 10:是 20:否

/**
* @brief 云机来源： 0：购买  1试用 2:免费激活码 3:免费活动抽奖 4：ar app注册 5：9.9元套餐年卡 ',
*/
@property (nonatomic, assign) NSInteger  sourceType;

/**
* @brief 云机类型： 1 计时套餐  0 普通云手机套餐 2 普通计时套餐 3:自动续费计时
*/
@property (nonatomic, assign) NSInteger  userCardType;

/**
* @brief  云手机推流编码   H264 H265
*/
@property (nonatomic,copy) NSString *videoCode;

/**
 * @brief 最近获取设备的时间
*/
@property (nonatomic,assign)double getDeviceTime;

/**
*
* @brief  云手机是否需要更新服务器上的图片
*/
@property (nonatomic, assign) BOOL needUpdateImageInSever;

/**
 * @brief 设备请求刷新缩略图的时间
*/
@property (nonatomic,assign)double deviceRefeshImageTime;

/**
* @brief 修改云机参数状态 1：开启 0：关闭 1：开启
*/
@property (nonatomic, assign) NSInteger modifyCardParamStatus;

/**
* @brief 虚拟定位状态  1：开启    0：关闭
*/
@property (nonatomic, assign) NSInteger virtualGpsStatus;

/**
* @brief ROOT开关状态 1：开启    0：关闭
*/
@property (nonatomic, assign) NSInteger rootSwitchStatus;

/**
* @brief 站点迁移状态 1：开启    0：关闭
*/
@property (nonatomic, assign) NSInteger siteTransferStatus;

/**
* @brief 一键换机状态 1：开启    0：关闭
*/
@property (nonatomic, assign) NSInteger oneButtonStatus;

/**
* @brief 云机摄像头状态 1：开启 0：关闭 2: 不可用（当前云手机摄像头不可用）
*/
@property (nonatomic, assign) NSInteger phoneCameraStatus;

/**
* @brief 自动管理续费状态 1：已购买 0：未购买
*/
@property (nonatomic, assign) NSInteger sign;

/**
* @brief  自动续费套餐周期（天）
*/
@property (nonatomic,assign) NSInteger period;

/**
* @brief 未被超分前的云手机分配的机房名称
*/
@property (nonatomic, copy) NSString *room;

/**
* @brief 增值服务使用记录 虚拟全部model标识符
*/
@property (nonatomic,assign) int isTrialAll;

/**
* @brief 业务通讯外网端口 （暂时未使用）
*/
@property (nonatomic, copy) NSString *internetBusinessPort;

/**
* @brief 板卡型号
*/
@property (nonatomic, copy) NSString *deviceBoard;

/**
* @brief 机房外网wss地址
*/
@property (nonatomic, copy) NSString *internetHttps;

/**
* @brief 机房外网ws地址
*/
@property (nonatomic, copy) NSString *internetHttp;

/**
* @brief 安卓卡内网ws地址
*/
@property (nonatomic, copy) NSString *localIp;

/**
* @brief 安卓卡校验token
*/
@property (nonatomic, copy) NSString *cardToken;

/**
* @brief 安卓系统  7、10
*/
@property (nonatomic, copy) NSString *androidVersion;

/**
* @brief  系统升级显示状态 1：开启 0：关闭
*/
@property (nonatomic,assign) int systemUpgradeStatus;

/**
* @brief  一键诊断 1：诊断中 0
*/
@property (nonatomic,assign) int oneKeyRepairStatus;

/**
* @brief 推流编码（H264、H265）
*/
@property (nonatomic, copy) NSString *pushFlowCode;

/**
* @brief 分辨率（720、1080）
*/
@property (nonatomic, copy) NSString *resolvingPower;

/**
* @brief 分辨率高度
*/
@property (nonatomic,assign) NSInteger  high;/*高度*/

/**
* @brief 分辨率宽度
*/
@property (nonatomic,assign) NSInteger  width;/*宽度*/

/**
* @brief 分辨率密度
*/
@property (nonatomic,assign) NSInteger  dpi;/*密度*/

/**
* @brief 设置分辨率时间
*/
@property (nonatomic,strong) NSDate  *dpiSetime;/*分辨率密度设置时间*/

/**
* @brief 一键修复剩余的次数
*/
@property (nonatomic,assign) NSInteger  oneKeyRepairResidueNum;

/**
* @brief vpn模式 0：自动 1：UDP 2:TCP
*/
@property (nonatomic,assign) NSInteger  vpnMode;

/**
* @brief 计时状态    0：停止计时 1：开始计时  2：等待进入状态
*/
@property (nonatomic,assign) NSInteger  timingStatus;

/**
* @brief 是否展示倒计时：0：不展示 1：展示（9.9套餐活动）
*/
@property (nonatomic,assign) NSInteger  isShowCountdown;

/**
* @brief 是否首次连接 0: 否 1：是（9.9套餐活动）
*/
@property (nonatomic,assign) NSInteger  isFirstConnect;

/**
* @brief 0：否 1：是
*/
@property (nonatomic,assign) NSInteger  isShowRule;

//是否开启虚拟键盘
- (BOOL)isOpenVirtualKey;

//是否开启专业模式
- (BOOL)isOpenUsageMode;

//是否开启左手
- (BOOL)isOpenLeftModel;

//是否打开ROOT开关
- (BOOL)isOpenRoot;

/**
 * @brief 增值服务限免Codes
 */
@property (nonatomic,strong) NSMutableArray *freeCodes;

/**
 * @brief 云手机功能Codes - visible
 */
@property (nonatomic,strong) NSMutableArray *visibleFuncCodes;


/**
 * @brief 云机业务状态 ：0：空闲 1：系统升级 2：套餐升级 3：非超分转换超分 4:连接队列 5：等待进入
 */
@property (nonatomic,assign) NSInteger  businessStatus;

/**
 * @brief 年卡下次发送时间
*/
@property (nonatomic, copy) NSString* nextSendTime;

/**
 * @brief 是否为转移的云机  1:是  0:不是
*/
@property (nonatomic,assign) NSInteger  transferStatus;

@end



@protocol CloudPhoneModel;

/**
`云手机列表相关
@discussion diskInfo     云手机列表
@discussion isTrial        是否有体验权限  0没有  1有
*/
@interface CloudPhoneDataModel : SuperModel

/**
 * @brief 云手机列表数据
 */
@property (nonatomic,strong) CloudPhoneModel *data;

@end


@protocol CloudPhoneModel;
/**
`云手机列表相关
@discussion diskInfo     云手机列表
@discussion isTrial        是否有体验权限  0没有  1有
*/
@interface CloudPhoneListModel : SuperModel

/**
 * @brief 云手机列表
 */
@property (nonatomic,strong) NSMutableArray <CloudPhoneModel>*diskInfo;

/**
 * @brief 增值服务限免Codes
 */
@property (nonatomic,strong) NSMutableArray *freeCodes;

/**
* @brief 是否有体验权限
*/
@property (nonatomic,assign) int isTrial;

@end

@protocol CloudPhoneListModel;

/**
`云手机列表相关
@discussion diskInfo     云手机列表
@discussion isTrial        是否有体验权限  0没有  1有
*/
@interface CloduPhoneListDataModel : SuperModel

/**
 * @brief 云手机列表数据
 */
@property (nonatomic,strong) CloudPhoneListModel *data;

@end


@protocol CloudPhoneModel;
/**
`续费云手机列表
*/
@interface CloudPhoneListForRenewModel : SuperModel

/**
 * @brief 云手机列表
 */
@property (nonatomic,strong) NSMutableArray <CloudPhoneModel>*data;

@end
NS_ASSUME_NONNULL_END

